Dependencies.require_or_load "execute_analysis"

class AnalysisController < ApplicationController
  include ExecuteAnalysis

  layout "gfdnavi"

  before_filter :login_required, :only => :save

  def index
    if session[:analysis]
      @analysis = session[:analysis]
    else
      @analysis = Analysis.new
      @analysis.user = session[:user]
      clear_diagram_files
      clear_temp_variables()
    end
    session[:analysis] = @analysis
    @variables = all_variables

    @function_input_nums, @functions = function_set(@analysis)
    @history = session[:history]
    if (diagrams = session[:diagrams])
      @diagrams = diagrams.sort.collect{|i,dia| diagram_to_dginfo(dia)}
    end
  end

  def load_history
    i = params[:id].to_i
    unless session[:history][i]
      render :update do |page|
          page.replace_html :messages, "operation failed. please reload the page."
      end
    end
    analysis = session[:history][i][0]
    session[:analysis] = analysis
    redirect_to(:action => "index")
  end

  def clear
    session[:analysis] = nil
    session[:history] = nil
    clear_variables
    clear_diagram_files
    redirect_to(:action => "index")
  end

  def clear_diagrams
    clear_diagram_files
    if request.xhr?
      render(:update){|page|
        page.replace_html :diagrams, ""
      }
    else
      redirect_to(:action => "index")
    end
  end

  def variables_selected
    unless request.xhr?
      clear
      return
    end
    session[:analysis] = variables_set(params["variables"])
    action_type_selected
  end

  def action_type_selected
    unless request.xhr?
      clear
      return
    end
    @analysis = session[:analysis] || Analysis.new
    @analysis.user = session[:user]
    @analysis.action_type = params[:action_type] if params[:action_type]
    if @analysis.action_type == 0 # draw
      if @analysis.dimensions.length >= 2
        @analysis.draw_type = 1 # tone and contour
      else
        @analysis.draw_type = 0 # line
      end
    end
    session[:analysis] = @analysis
    messages = ""
    if @analysis.variables.length == 0
      @analysis.variable_clear
    elsif @analysis.dimensions.length == 0
      messages = "all selected variables must have the same dimensions"
    end

    if Analysis::ACTION_TYPE[@analysis.action_type] == "analysis"
      @function_input_nums, @functions = function_set(@analysis)
    end
    render :update do |page|
      page.replace_html :messages, messages
      page.replace_html :dimensions_setting, render(:partial => "dimension_option")
      case Analysis::ACTION_TYPE[@analysis.action_type]
      when "draw"
        page.replace_html :draw_settings, render(:partial => "draw_settings")
        page.replace_html :popular_diagrams, render(:partial => "popular_diagrams")
      when "analysis"
        page.replace_html :analysis_settings, render(:partial => "analysis_settings")
        page.replace_html :popular_diagrams, ""
      end

      page.replace_html :script, <<"EOF"
<script>
  action_type = #{@analysis.action_type};
  executable = #{@analysis.dimensions.length > 0};
  afterCallBack();
</script>
EOF
    end

  end

  def function_input_num_selected
    unless request.xhr?
      clear
      return
    end
    @analysis = session[:analysis] || Analysis.new
    @analysis.user = session[:user]
    @analysis.input_num = params[:function_input_num]
    @function_input_nums, @functions = function_set(@analysis)
    render :update do |page|
      page.replace_html :analysis_function_settings, render(:partial => "analysis_function_settings")
    end
  end


  def execute
    params_analysis = params[:analysis] || Hash.new
    if params_analysis[:anim]=="1" && (anim = params[:anim])
      val = anim["val"]
      region_org = params_analysis["region"].dup
      anim_dim = params_analysis["anim_dim"]
      params_analysis["region"].delete(anim_dim)
      params_analysis["region"][anim_dim] = {"min"=> val}
      anim = true
    else
      anim = false
    end
    action_type = params[:action_type]
    if request.xhr?
      analysis = session[:analysis]
      unless analysis
        render :update do |page|
          page.replace_html :messages, "operation failed. please reload the page."
        end
        return
      end
    else
      analysis = variables_set(params["variables"])
      analysis.draw_keep = false
      analysis.draw_share = false
      analysis.anim = anim
      analysis.action_type = Analysis::ACTION_TYPE.index(action_type)
    end
    analysis.attributes = params_analysis
    session[:analysis] = analysis
    session[:history] ||= AnalysisHistory.new
    session[:history].push YAML.load(analysis.to_yaml)
    case action_type
    when "draw"
      execute_draw(analysis, anim)
      if anim
        analysis.region = region_org
        session[:analysis] = analysis
      end
      return
    when "analysis"
      if request.xhr?
        execute_analysis(analysis)
        return
      end
    end
    raise "invalid action (#{action_type})"
  end

  def download_diagram_script_and_data
    id = params[:id]
    diagram = get_temp_diagram(id)
    if diagram
      viz = diagram[:vizshot]
      if viz
        res = viz_to_script_and_data(viz, work_dir)
        if res[0]
          send_file res[1], :filename => "gfdnavi.tar.gz"
          return
        else
          message = res[1]
        end
      else
        message = "cannot find data to download"
      end
    else
      message = "cannot find diagram"
    end
    flash[:messages] = "failed to download<br/>" + message
    redirect_to :action => "index"
  end

  def download_data
    id = params[:id]
    var = get_temp_variable(id)
    if var
      name = var.file_and_variable_names
      if name
        send_file name[0], :filename => "gfdnavi.nc"
        return
      end
    end
    render :update do |page|
      page.replace_html :messages, "cannot download data"
    end
  end

  def save
    if GFDNAVI_DISABLE_USER || !(user = session[:user])
      redirect_to :action => "index"
      return
    end
    if request.get?
      @id = params[:id]
      @type = params[:type]
      @groups = Group.find_for_user(user)
      @user = user
      case @type
      when "diagram"
        @suffix = "png"
      when "data"
        @name = get_temp_variable(@id).name
        @suffix = "nc"
      end
      render
      return
    end
    id = params[:id]
    type = params[:type]
    name = params[:name]
    path = params[:path]
    description = params[:description]
    keywords = params[:keywords]
    group = params[:group]
    case type
    when "diagram"
      var = Variable.new
      var.gphysizable = false
      diagram = get_temp_diagram(id)
      viz = diagram[:vizshot]
      ids = viz.get_variables.collect{|v|
        if Variable===v && ! v.id
          from = v.file_and_variable_names[0]
          to, pa = auto_file_path("data")
          File.move(from, to)
          v.path = pa + Variable::DIR_VAR_DELIMITER + v.name
          v.save!
        else
          v = Variable===v ? v : Variable.find(v)
        end
        var.references_tmp << v
        v.id
      }
      var.keyword_attributes.build(:name => "vizshot", :value => viz.to_yaml)
      path = "/#{path}.png"
      from = diagrams_dir+"/"+diagram[:fname]
    when "data"
      var = get_temp_variable(id)
      var.gphysizable = true
      path = "/#{path}.nc#{Variable::DIR_VAR_DELIMITER}#{name}"
      from = var.file_and_variable_names[0]
    end
    var.name = name
    var.path = "/usr/#{user.login}#{path}"
    var.user = user
    var.group = Group.find(:first, :conditions => ["name = ? and user_id = ?", group, user.id])
    to = var.file_and_variable_names[0]
    File.makedirs( File.dirname(to) )
    var.keyword_attributes.build(:name => "description", :value => description) unless description == ""
    keywords.each{|k,v|
      if Hash === v && (name=v["name"])!="" && (val=v["value"])!=""
        if val.to_i.to_s == val
          val = NArray[val.to_i]
        elsif val.to_f.to_s == val
          val = NArray[val.to_f]
        end
        var.keyword_attributes.build(:name => name, :value => val)
      end
    }
    if var.save
      messages = "successfully saved"
      case type
      when "diagram"
        File.copy( from, to )
        diagram[:saved] = true
      when "data"
        File.move( from, to )
        session[:temp_variables_list].delete var
        session[:variables_list].push var
      end
    else
      messages = "failed to save<br/>"
      messages += var.errors.full_messages.join("<br/>") if var.errors
    end
    flash[:messages] = messages
    redirect_to :action => :index
  end

  def delete_diagram
    id = params[:id].to_i
    if (diagram = session[:diagrams][id])
      session[:diagrams].delete(id)
      fname = diagrams_dir + "/" + diagram[:fname]
      File.delete(fname)
    end
    render :nothing => true
  end

  def pile_up
    return unless request.xhr?
    lower_id = params[:lower].to_i
    upper_id = params[:upper].to_i
    if (diagrams = session[:diagrams])
      lower = diagrams[lower_id]
      upper = diagrams[upper_id]
      if lower && upper
        viz = lower[:vizshot].add(upper[:vizshot])
        vizs_to_diagram([viz], true, session[:analysis].draw_share)
        return
      end
    end
    render :nothing => true
  end

  private
  def all_variables
    vars = session[:variables_list] || []
    variables = vars.collect{|v|
      attrs = v.keyword_attributes.collect{|attr| [attr.name, attr.value]}
      selected = @analysis.variables.include?(v)
      [v.id, v.name, attrs, selected, false, v.path]
    }
    temp_vars = session[:temp_variables_list] || []
    for i in 0...temp_vars.length
      v = temp_vars[i]
      attrs = v.keyword_attributes.collect{|attr| [attr.name, attr.value]}
      selected = @analysis.variables.include?(v)
      variables.push [i, v.name, attrs, selected, true, v.path]
    end
    return variables
  end

  def function_set(analysis)
    input_nums = Array.new
    nvars = analysis.variables.length
    user = session[:user]
    nvars.times{|ii|
      i = ii+1
      if (nvars.to_f/i).to_i*i == nvars && Function.find_for_user(user, :first, :conditions => ["input_num=?", i])
        input_nums.push(i)
      end
    }
    if input_nums.length > 0
      analysis.input_num = input_nums[-1] unless analysis.input_num
      functions = Function.find_for_user(user, :all, :conditions => ["input_num = ?", analysis.input_num], :order => [:name])
    else
      functions = Array.new
    end
    return [input_nums, functions]
  end

#draw
  def execute_draw(analysis, anim=false)
    res = get_diagram(analysis, anim||nil)
    unless res[0]
      if request.xhr?
        render :update do |page|
          page.replace_html :messages, res[1]
        end
      else
        render :text => res[1]
      end
      return
    end
    diagrams = res[1]
    keep = res[2]
    draw_or_cache = res[3]
    if keep
      session[:diagrams] ||= Hash.new
    else
      session[:diagrams] = Hash.new
    end
    diagrams.each{|diagram|
      id = diagram[:id]
      session[:diagrams][id] = diagram
    }
    show_diagram(diagrams, keep, draw_or_cache, anim)
  end

  def show_diagram(diagrams, keep, draw_or_cache, anim)
    dginfos = diagrams.collect{|diagram| diagram_to_dginfo(diagram)}
    if request.xhr?
      @analysis = session[:analysis]
      render(:update){|page|
        page.replace_html :messages, ""
        if GFDNAVI_BENCHMARK && !anim
          ids = dginfos.collect{|di| di[2]}
          html = host_information_table
          html +=<<-"EOS"
            Total number of diagrams is #{ids.length}<br/>
            Cached diagram: #{draw_or_cache[1]}<br/>
            Drawed diagram: #{draw_or_cache[0]}
          EOS
          page << "benchMark.set(['#{ids.join('\',\'')}'], '#{escape_javascript(html)}');"
          @benchmark = true
        end
        @anim = anim
        dginfos.length.times{|i|
          @diagram = dginfos[i]
          if keep || i>0
            page.insert_html :bottom, :diagrams, render(:partial => "diagram")
          else
            page.replace_html :diagrams, render(:partial => "diagram")
          end
          if anim
            page << "anim.diagrams.push($('#{@diagram[0]}_table'));"
            page << "anim.next();"
          end
        }
        @history = session[:history]
        page.replace_html :history, render(:partial => "history")
      }
      return
    else
      @paths = dginfos.collect{|at| at[1]}
      render :action => :show_diagram
    end
  end

# analysis
  def execute_analysis(analysis)
    vars, messages = create_new_variable(analysis, work_dir)
    if vars
      session[:temp_variables_list] ||= Array.new
      vars.each{|var| session[:temp_variables_list].push var}
      @analysis = analysis
      @variables = all_variables
      render :update do |page|
        page.replace_html :messages, messages
        page.replace_html :variables_body, render(:partial => "variables")
        if GFDNAVI_BENCHMARK
          html = host_information_table
          html +=<<-"EOS"
            Number of created variable is #{vars.length}.<br/>
          EOS
          page << "benchMark.set(null, '#{escape_javascript(html)}');"
          page << "benchMark.complete();"
        end
        @history = session[:history]
        page.replace_html :history, render(:partial => "history")
      end
      return
    else
      render :update do |page|
        page.replace_html :messages, messages
      end
    end
  end


#common
  def get_temp_variable(id)
    variables = session[:temp_variables_list]
    if variables
      return variables[id.to_i]
    else
      return nil
    end
  end

  def get_temp_diagram(id)
    diagrams = session[:diagrams]
    if diagrams
      return diagrams[id.to_i]
    else
      return nil
    end
  end

  def clear_temp_variables
    files = Dir["#{work_dir}/*.nc"].each do |file|
      File.delete(file)
    end
    session[:temp_variables_list] = nil
  end

  def clear_variables
    clear_temp_variables()
    session[:variables_list] = nil
  end

  def auto_file_path(type)
    case type
    when "data"
      suffix = "nc"
    when "diagram"
      suffix = "png"
    end
    path = user_path+"/auto"
    FileUtils.makedirs(path) unless File.exist?(path)
    last = Dir["#{path}/gfdnavi_\d+\.#{suffix}"][0]
    if last
      last =~ /gfdnavi_(\d+)\./
      num = $1.next
    else
      num = "0000"
    end
    fname = "gfdnavi_#{num}.#{suffix}"
    full_path = "#{path}/#{fname}"
    path = "/usr/#{session[:user].login}/auto/#{fname}"
    return [full_path, path]
  end



end
