class Knowledge < ActiveRecord::Base
  has_many :knowledge_figures, :dependent => :destroy

  validates_uniqueness_of :knowledge_path
  validates_length_of :title, :knowledge_path, :within => 0..400
  validates_length_of :textbody, :within => 0..100000
  validates_format_of :knowledge_path, :with => /\w|-|\//, :message => "knowledge_path is invalid"

  #def self.find(*args)
  #  raise NoMethodError, "This method is disabled. Use find_for_user"
  #end

  def self.find_for_user(user, *args)
    args = [:all] if args.length == 0
    if user && user.login == "root"
      return self.find(*args)
    end

    grp_cond = Group.group_conditions(user)

    if args.length == 1
      args.push({:conditions => grp_cond})
    elsif args[1].is_a?(Hash)
      if (cond = args[1][:conditions])
        if Array === cond
          args[1][:conditions] = ( ['( '+ cond[0] +' ) AND ' + grp_cond ] + cond[1..-1] )
        else
          args[1][:conditions] = '( '+ cond +' ) AND ' + grp_cond
        end
      else
        args[1][:conditions] = grp_cond
      end
    else
      # args must be IDs --> put them in an array
      args.each do |x|
        if !x.is_a?(Integer)
          raise ArgumentError,
            "ID must be integers, but #{x.inspect} isn't. Or you need to give"+
            "a Hash as the 2nd argument, which is now #{args[1].inspect}."
        end
      end
      args = [ args, {:conditions =>grp_cond} ]
    end

    return self.find(*args)
  end

  def self.find_for_owner(user, *args)
    args = [:all] if args.length == 0
    if user.login == "root"
      return self.find(*args)
    end

    case user
    when User
      owner_cond = "user_id = #{user.id}"
    ## #//-- for future extention -->
    ## when String
    ##   owner_cond = self.sanitize_sql(["guest_name = %s",user])
    ## #<-- for future extention //
    else
      raise ArgumentError, "Unexpected user type"
    end

    if args.length == 1
      args.push({:conditions => owner_cond})
    elsif args[1].is_a?(Hash)
      if (cond = args[1][:conditions])
        if Array === cond
          args[1][:conditions] = ( ['( '+ cond[0] +' ) AND ' + owner_cond ] + cond[1..-1] )
        else
          args[1][:conditions] = '( '+ cond +' ) AND ' + owner_cond
        end
      else
        args[1][:conditions] = owner_cond
      end
    else
      # args must be IDs --> put them in an array
      args.each do |x|
        if !x.is_a?(Integer)
          raise ArgumentError,
            "ID must be integers, but #{x.inspect} isn't. Or you need to give"+
            "a Hash as the 2nd argument, which is now #{args[1].inspect}."
        end
      end
      args = [ args, {:conditions =>owner_cond} ]
    end

    return self.find(*args)
  end

end
